#ifndef CONNECTOPERATION_H
#define CONNECTOPERATION_H

#include "peony-core_global.h"

#include <QObject>

#include <gio/gio.h>
#include <memory>

namespace Peony {

class ConnectServerDialog;
class ConnectServerLogin;

class PEONYCORESHARED_EXPORT ConnectOperation : public QObject
{
    Q_OBJECT
public:
    explicit ConnectOperation(QObject *parent = nullptr);
    ~ConnectOperation();

Q_SIGNALS:
    void updateLocationUri(const QString& uri);

protected:
    static GAsyncReadyCallback mount_enclosing_volume_callback(GFile *volume,
                                                               GAsyncResult *res,
                                                               ConnectOperation *p_this);

    static void
    aborted_cb (GMountOperation *op,
                ConnectOperation *p_this);

    static void
    ask_question_cb (GMountOperation *op,
                     char *message,
                     char **choices,
                     ConnectOperation *p_this);

    static void
    ask_password_cb (GMountOperation *op,
                     const char      *message,
                     const char      *default_user,
                     const char      *default_domain,
                     GAskPasswordFlags flags,
                     ConnectOperation *p_this);

private:
    GFile *m_volume = nullptr;
    GMountOperation *m_op = nullptr;

    ConnectServerDialog *m_connectServerDialog = nullptr;
    ConnectServerLogin *m_connectServerLogin = nullptr;
};

}
#endif // CONNECTOPERATION_H
