/*
 * Peony-Qt's Library
 *
 * Copyright (C) 2020, KylinSoft Co., Ltd.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: Yue Lan <lanyue@kylinos.cn>
 *
 */

#ifndef GLOBALSETTINGS_H
#define GLOBALSETTINGS_H

#include <QObject>
#include <QSettings>
#include <QMutex>
#include <QDBusInterface>
#include <QJsonObject>

#include "peony-core_global.h"
#include "convenient-macros.h"

#include <gio/gio.h>

#define FORBID_THUMBNAIL_IN_VIEW    "do-not-thumbnail"
#define SHOW_HIDDEN_PREFERENCE      "showHiddenFile"
#define SORT_CHINESE_FIRST          "chinese-first"
#define SORT_FOLDER_FIRST           "folder-first"
#define SORT_ORDER                  "sort-order"
#define SORT_COLUMN                 "sort-column"
#define RESIDENT_IN_BACKEND         "resident"
#define LAST_DESKTOP_SORT_ORDER     "last-desktop-sort-order"
#define DESKTOP_SORT_ORDER          "desktop-sort-order"
#define ALLOW_FILE_OP_PARALLEL      "allow-file-op-parallel"
#define DEFAULT_WINDOW_SIZE         "default-window-size"
#define DEFAULT_SIDEBAR_WIDTH       "default-sidebar-width"
#define SHOW_TRASH_DIALOG           "showTrashDialog"
#define INIT_FOR_FIRST_TIME         "initForFirstTime"  /* org.ukui.peony.settings 首次初始化标志 */
#define SHOW_FILE_EXTENSION         "showFileExtension" /* 显示文件扩展名 */
#define SEND_URIS_OF_COPY_DSPS      "sendUrisOfCopyDsps"  /* send src and dest uris of copy dsps files to wps,默认值为false */
#define DOC_IS_OCCUPIED_BY_WPS      "docIsOccupiedByWps" /* 是否开启wps占用文件监控功能 */
#define SHOW_NETWORK                "showNetwork"
#define SHOW_IN_NEW_WINDOW          "showInNewWindow"
#define DISABLED_EXTENSIONS         "disabledExtensions"
#define SHOW_CREATE_TIME            "showCreateTime"
#define SHOW_RELATIVE_DATE          "showRelativeDate"
#define SHOW_SHARE_PROPERTIES       "showShareProperties"

#define DEFAULT_GRID_SIZE           "default-grid-size"

#define USE_GLOBAL_DEFAULT_SORTING "useGlobalDefaultSorting"

#define DEFAULT_VIEW_ID             "directory-view/default-view-id"
#define DEFAULT_VIEW_ZOOM_LEVEL     "directory-view/default-view-zoom-level"

#define REMOTE_SERVER_REMOTE_IP     "remote-server/favorite-ip"
//#define REMOTE_SERVER_CONNECT_IP   "remote-server/connecte-ip"
//gsettings
#define PERSONAL_EFFECT_SCHEMA       "org.ukui.control-center.personalise"
#define PERSONAL_EFFECT_ENABLE       "effect"
#define PERSONAL_EFFECT_TRANSPARENCY "transparency"
#define SIDEBAR_BG_OPACITY          "sidebar-bg-opacity"

#define FONT_SETTINGS                "org.ukui.style"
#define UKUI_SEARCH_SCHEMAS          "org.ukui.search.settings"
#define SEARCH_METHOD_KEY            "fileIndexEnable"
#define CONTENT_INDEX_ENABLE         "contentIndexEnable"
#define CONTENT_INDEX_ENABLE_OCR     "contentIndexEnableOcr"

//difference between Community version and Commercial version
#define COMMERCIAL_VERSION          true
//TEMPLATES standard path
#define TEMPLATES_DIR               "templates-dir"

//Control the display of desktop standard icons
#define DISPLAY_STANDARD_ICONS       "displayStandardIcons"
#define HOME_ICON_VISIBLE            "homeIconVisible"
#define TRASH_ICON_VISIBLE           "trashIconVisible"
#define COMPUTER_ICON_VISIBLE        "computerIconVisible"

//control the mobile device trash file issue, if be true can trash mobile files
#define TRASH_MOBILE_FILES            "trashMobileFiles"

//Control the display of right menu terminal
#define SHOW_OPEN_TERMINAL           "showOpenTerminal"

//Control start peony
#define ENABLE_START_PEONY           "enableStartPeony"

//Control double click desktop file
#define ENABLE_DOUBLE_CLICK_DESKTOP  "enableDoubleClickDesktop"

//Control file operation of shortcut keys, such as Ctrl+C、Ctrl+V、Ctrl+A、Delete、Shift+Delete、Ctrl+X
#define ENABLE_SHORTCUT_KEYS         "enableShortcutKeys"

//Control rignt menu in desktop file
#define ENABLE_DESKTOP_FILE_MENU     "enableDesktopFileMenu"

//Control rignt menu in desktop background
#define ENABLE_DESKTOP_BACKGROUND_MENU     "enableDesktopBackgroundMenu"

//Control rignt menu in peony
#define ENABLE_PEONY_RIGHT_MENU     "enablePeonyRightMenu"

//Stores multi combobox indexs
#define MULTI_SEARCH_INDEXS     "multiSearchIndexs"

// control center
#define UKUI_CONTROL_CENTER_PANEL_PLUGIN            "org.ukui.control-center.panel.plugins"                 // schema
#define UKUI_CONTROL_CENTER_PANEL_PLUGIN_TIME       "org.ukui.control-center.panel.plugins.time"            // time format key, value is '12' or '24'
#define UKUI_CONTROL_CENTER_PANEL_PLUGIN_DATE       "org.ukui.control-center.panel.plugins.date"            // date format key, value is cn or en

// desktop
#define DESKTOP_USE_AUTO_LAYOUT      "desktopUseAutoLayout"

// guestos machine
#define IS_GUESTOS_MACHINE           "isGuestOSMachine"

#ifdef KY_SDK_DATE
#define SDK_DATE_SERVER_PATH "/com/kylin/kysdk/Date"
#define SDK_DATE_SERVER_SERVICE "com.kylin.kysdk.DateServer"
#define SDK_DATE_SERVER_INTERFACE "com.kylin.kysdk.DateInterface"
#endif

// Configuration paths and schema
#define CONFIG_BASE_PATH          "/usr/share/ukui-config"
#define PEONY_SCHEMA_ID           "org.ukui.peony.settings"
#define PEONY_SCHEMA_PATH         "/org/ukui/peony/settings/"
#define PEONY_CONFIG_NAME         "org.ukui.peony.settings"
#define PEONY_SETTINGS_KEY        "org.ukui.peony.settings"

class QGSettings;

namespace Peony {

/*!
 * \brief The GlobalSettings class
 * \details
 * this class provide a global settings for peony-qt.
 * it most be used to save and load preferences of peony-qt, such as default view,
 * sort type, etc.
 *
 * you can also save another kind of datas using by extensions. such as enable properties.
 * this class instance is shared in both peony-qt and its plugins.
 */
class PEONYCORESHARED_EXPORT GlobalSettings : public QObject
{
    Q_OBJECT
public:
    static GlobalSettings *getInstance();
    const QVariant getValue(const QString &key);
    bool isExist(const QString &key);
    bool initDateFormatDBus();
    void sendNotifyMessage(const QString &msg);

    bool getShowCreateTime() const;
    bool isDesktopStartUp() const;
    void setDesktopStartUp(bool startUp);/* 桌面启动和结束时使用，谨慎调用 */
    QString getlocalDiskShowUnmountOptStr();

Q_SIGNALS:
    void valueChanged(const QString &key);
    void updateHiddenFile(const QString &fileName);
    void signal_updateRemoteServer(const QString& server, bool add);
    void updateShortDataFormat(const QString &fileName);
    void updateLongDataFormat(const QString &fileName);

public Q_SLOTS:
    void setValue(const QString &key, const QVariant &value);
    void reset(const QString &key);
    void resetAll();
    void setTimeFormat(const QString &value);
    void setDateFormat(const QString &value);
    void sendShortDataFormat(const QString &format);
    void sendLongDataFormat(const QString &format);
    QString getSystemTimeFormat();
    QString transToSystemTimeFormat(guint64 mtime, bool longFormat=false);

    /*!
     * \brief 通过GSetting保存设置
     * \param key
     * \param value
     */
    void setGSettingValue(const QString &key, const QVariant &value);

    /*!
     * \brief forceSync
     * \param key
     * \details
     * use force sync to ensure the settings key-value and cached map's data
     * keep same. this may be used in out progress, such as peony-qt-desktop.
     */
    void forceSync(const QString &key = nullptr);
    void slot_updateRemoteServer(const QString& server, bool add);

    bool isGuestOSMachine();

    /**
     * @brief Slot for handling configuration file changes
     * @param username The username
     * @param configName The configuration name
     * @param configPath The configuration file path
     */
    void onConfigFileChanged(const QString &username, const QString &configName,
                            const QString &configPath);

private:
    explicit GlobalSettings(QObject *parent = nullptr);
    ~GlobalSettings();

    /**
     * @brief Initialize safe manage control configuration sources
     */
    void initManageControl();

    /**
     * @brief Load and parse JSON configuration file
     *
     * This function attempts to load and parse a JSON configuration file from the specified path.
     * If the file cannot be opened, parsed, or contains invalid data, the JSON cache will be cleared.
     *
     * The JSON structure should follow the format:
     * {
     *     "org.ukui.peony.settings": {
     *         "setting-key1": "value1",
     *         "setting-key2": "value2"
     *     }
     * }
     *
     * @param configPath The full path to the JSON configuration file
     * @return bool Returns true if the configuration was successfully loaded and parsed,
     *              false if any error occurred during the process
     *
     * @note On any failure (file not found, parse error, empty content),
     *       the internal JSON cache (m_jsonCache) will be cleared
     *
     * @see updateConfigCache()
     */
    bool loadJsonConfig(const QString &configPath);

    /**
     * @brief Initialize DBus connection
     */
    void initDBus();

    /**
     * @brief Convert JSON configuration key to internal key format
     * @param jsonKey The key in JSON format
     * @return QString The converted internal key
     */
    QString convertJsonKeyToInternalKey(const QString &jsonKey);

    /**
     * @brief Update configuration cache and emit signals for changed values
     * @param newConfig The new configuration object
     */
    void updateConfigCache(const QJsonObject &peonySettings);

    /**
     * @brief Construct configuration file path
     * @param username The username
     * @return QString The complete configuration file path
     */
    QString constructConfigPath(const QString &username) const;

    QMap<QString, QVariant>     m_jsonCache;       ///< Cache for JSON configuration values
    QString                     m_currentConfigPath;

    QSettings*                  m_settings;
    QMap<QString, QVariant>     m_cache;

    QGSettings*                 m_gsettings = nullptr;
    QGSettings*                 m_peonyGSettings = nullptr;
    QGSettings*                 m_control_center_plugin = nullptr;
    QGSettings*                 m_peony_gsettings  = nullptr;

    QMutex                      m_mutex;

    QString                     m_date_format = "";
    QString                     m_time_format = "";
    QString                     m_system_time_format  = "";
    QString                     m_localDiskShowUnmountOptStr; /* 长城的机器默认值是false，其它默认值为true，设置项文案“本地磁盘显示卸载选项”;设置项勾选后，右键卸载按钮和三角形按钮显示;相关需求31180 */

    QDBusInterface*             mDbusDateServer = nullptr;
    bool                        m_showCreateTime = false;
    bool                        m_showRelativeTime = false;
    bool                        m_isDesktopStartUp = false; /* 桌面进程启动中 */
};

}

#endif // GLOBALSETTINGS_H
